#![feature(str_split_remainder)]
use chrono::prelude::*;
use derive_more::Constructor;
use markdown::{to_html_with_options, CompileOptions, Options};
use regex::Regex;
use std::collections::HashMap;
use std::ffi::OsString;
use std::fs::{self, read_dir, read_to_string};
use std::path::PathBuf;
use tera::{Context, Tera};

pub fn read_src_files(src_dir: &str) -> Vec<SrcMD> {
    let mut files: Vec<SrcMD> = Vec::new();
    for file in read_dir(src_dir).expect("Cant read src dir") {
        let file = file.unwrap();
        let frontmatter = get_frontmatter(read_to_string(file.path()).unwrap()).0;
        let title = frontmatter
            .get("title")
            .expect("Error getting field: title from frontmatter");
        let date = frontmatter
            .get("date")
            .expect("Error getting field: date from frontmatter");
        files.push(SrcMD::new(
            file.path(),
            title.to_string(),
            DateTime::parse_from_rfc3339(date).unwrap(),
            file.path().file_stem().unwrap().to_os_string(),
        ))
    }

    files
}

pub fn write_to_fs(html: String, output_dir: &String, file_name: &String) {
    fs::write(format!("{output_dir}/{file_name}.html"), html)
        .unwrap_or_else(|_| panic!("Error writing {file_name}"));
}

#[derive(Constructor, Debug)]
pub struct SrcMD {
    pub path: PathBuf,
    pub title: String,
    pub date: DateTime<FixedOffset>,
    pub file_name: OsString,
}

pub fn generate_blog_entry(
    markdown: String,
    template_dir: &String,
    frontmatter: &mut HashMap<String, String>,
) -> Option<(String, String)> {
    let markdown = markdown.clone();
    let mut tera = Tera::new(&format!("{template_dir}/*")).unwrap();
    tera.autoescape_on(vec![]);

    let html_markdown = to_html_with_options(
        &markdown,
        &Options {
            compile: CompileOptions {
                allow_dangerous_html: true,
                allow_dangerous_protocol: true,
                ..CompileOptions::default()
            },
            ..Options::gfm()
        },
    )
    .unwrap();

    frontmatter.insert("blog_content".to_string(), html_markdown.clone());

    let context = Context::from_serialize(&frontmatter).unwrap();

    let templated_html = tera.render(frontmatter.get("template")?, &context).unwrap();
    Some((templated_html, html_markdown))
}

pub fn get_frontmatter(markdown: String) -> (HashMap<String, String>, String) {
    let re_key_value = Regex::new(r"(?msU)---(.*)---(?:\n)").unwrap();

    let key_value_string = re_key_value
        .find(markdown.as_str())
        .expect("Can't find frontmatter in markdown");

    let content_markdown = re_key_value
        .replace(markdown.clone().as_str(), "")
        .to_string();

    let mut key_value: HashMap<String, String> = HashMap::new();

    for line in key_value_string.as_str().lines() {
        if line == "---" {
            continue;
        }
        let second_value = {
            let mut line_iter = line.split(':');
            line_iter.next();

            line_iter.remainder().unwrap().trim().to_string()
        };
        key_value.insert(
            line.split(':')
                .collect::<Vec<&str>>()
                .first()
                .unwrap()
                .trim()
                .to_string(),
            second_value,
        );
    }

    (key_value, content_markdown)
}
